// /Copyright 2003-2005 Arthur van Hoff, Rick Blair
// Licensed under Apache License version 2.0
// Original license LGPL

package javax.jmdns.impl;

import com.elitecom.avcontroller.discovery.boujour.HXServiceResolver;
import com.elitecom.avcontroller.utils.InteAddressUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.jmdns.ServiceEvent;
import javax.jmdns.impl.constants.DNSConstants;
import javax.jmdns.impl.constants.DNSRecordType;
import java.io.IOException;
import java.net.InetAddress;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

// REMIND: multiple IP addresses

/**
 * mDNS implementation in Java.
 *
 * @author Arthur van Hoff, Rick Blair, Jeff Sonstein, Werner Randelshofer, Pierre Frisch, Scott Lewis, Kai Kreuzer, Victor Toni
 */
public class HxJmDNSImpl extends JmDNSImpl {
    private static Logger logger = LoggerFactory.getLogger(JmDNSImpl.class.getName());

    /**
     * Create an instance of JmDNS and bind it to a specific network interface given its IP-address.
     *
     * @param address IP address to bind to.
     * @param name    name of the newly created JmDNS
     * @throws IOException
     */
    public HxJmDNSImpl(InetAddress address, String name) throws IOException {
        super(address, name);
    }
    private HXServiceResolver resolver;

    public HXServiceResolver getResolver() {
        return resolver;
    }

    public void setResolver(HXServiceResolver resolver) {
        this.resolver = resolver;
    }

    /**
     * Create an instance of JmDNS and bind it to a specific network interface given its IP-address.
     *
     * @param address               IP address to bind to.
     * @param name                  name of the newly created JmDNS
     * @param threadSleepDurationMs time in milliseconds that the JmDNS listener thread should sleep between multicast receives
     * @throws IOException
     */
    public HxJmDNSImpl(InetAddress address, String name, long threadSleepDurationMs) throws IOException {
        super(address, name, threadSleepDurationMs);
    }
    @Override
    public void startServiceResolver(String type) {
        if (resolver!=null){
            resolver.run();
        }
    }
    private boolean isOlderThanOneSecond(DNSRecord dnsRecord, long timeToCompare) {
        return (dnsRecord.getCreated() < (timeToCompare - DNSConstants.FLUSH_RECORD_OLDER_THAN_1_SECOND*1000));
    }
    /**
     * Handle an incoming query. See if we can answer any part of it given our service infos.
     *
     * @param in
     * @param addr
     * @param port
     * @exception IOException
     */
    void handleQuery(DNSIncoming in, InetAddress addr, int port) throws IOException {
//        logger.info("handleQuery..........");
    }
    void handleResponse(DNSIncoming msg) throws IOException {
//        logger.info("handleResponse..........");
        String types="";
        if (msg._answers!=null && msg._additionals!=null){
            DNSRecord.Service srv=null;
            DNSRecord.IPv4Address address =null;
            for (DNSRecord additional : msg._additionals) {
                types+=additional.getRecordType()+":";
                if (additional.getRecordType()==DNSRecordType.TYPE_SRV ){
                    srv=( DNSRecord.Service)additional;
                }if (additional.getRecordType()==DNSRecordType.TYPE_A ){
                    address=( DNSRecord.IPv4Address)additional;
                }
            }

            String srvname="";
            String aaaaadress="";
            if (srv!=null){
              srvname=srv.getName();
            }
            if (address!=null){
                aaaaadress=address._addr.getHostAddress()  +"," +address.toString();
            }
//            logger.info("mdns srv {},aaaa {} ,types:{}, name :{}, add:{}",srv==null,address==null,types,srvname,aaaaadress);
            if (srv!=null) {
                if (address!=null) {
                    HxService hxsrv = new HxService(srv, address._addr.getHostAddress());
                    handleRecord(hxsrv);
                }else {
                    handleRecord(srv);
                }
            }
        }
    }
    void handleRecord(DNSRecord record, long now) {
//        logger.info("handleRecord..........");
    }



    void handleRecord(DNSRecord record) {

        if (InteAddressUtil.isMyIp(record.getRecordSource())){
//            logger.info("handleRecord  from {},{} ,skip",record.getRecordSource().getHostAddress(), InteAddressUtil.isMyIp(record.getRecordSource()));
            return;
        }
        // notify the listeners
        record.setTTL(10);
//            this.updateRecord(now, record, Operation.Add);
        if (DNSRecordType.TYPE_PTR.equals(record.getRecordType())
                || ( DNSRecordType.TYPE_SRV.equals(record.getRecordType()))){
            ServiceEvent event = record.getServiceEvent(this);
            List<ListenerStatus.ServiceListenerStatus> list = _serviceListeners.get(event.getType().toLowerCase());
            final List<ListenerStatus.ServiceListenerStatus> serviceListenerList;
            if (list != null) {
                synchronized (list) {
                    serviceListenerList = new ArrayList<ListenerStatus.ServiceListenerStatus>(list);
                }
            } else {
                serviceListenerList = Collections.emptyList();
            }
            for (final ListenerStatus.ServiceListenerStatus listener : serviceListenerList) {
                listener.getListener().serviceAdded(event);

            }
        }
    }

    /**
     * Send an outgoing multicast DNS message.
     *
     * @param out
     * @exception IOException
     */
    public void send(DNSOutgoing out) throws IOException {
        List<DNSQuestion> questions = out._questions;
        if (questions!=null){
            for (DNSQuestion question : questions) {
                if (!DNSRecordType.TYPE_PTR.equals( question.getRecordType())){
//                    logger.info("send ....q type {}  skip",question.getType());
                    return;
                }


            }
        }
        if (out._answers!=null&&out._answers.size()>0){
//            logger.info("send .... {} skip",out._answers.size());
        }else{
            super.send(out);
        }

    }
}
